@@ -52,7 +52,7 @@ class TotalScore(db.Model):
     """Holds the integer `value` of the total score for votes on a specific
     `content_object`. The `value` is updated whenever a Vote object is created,
     modified or deleted. Don't alter it directly."""
-    value = db.IntegerField(verbose_name=_("value"), default=0)
+    value = db.IntegerField(verbose_name=_("value"), default=0, db_index=True)
     content_type = db.ForeignKey(ContentType, verbose_name=_("Content type"),
         related_name="%(app_label)s_%(class)s_scores")
     object_id = db.IntegerField(verbose_name=_("Content type instance id"),
@@ -70,15 +70,13 @@ class TotalScore(db.Model):
         the specified model (and optionally: instance)."""
         if not ct:
             ct = ContentType.objects.get_for_model(model)
-        kwargs = {'content_type': ct}
+        kwargs = {'total_score__content_type': ct}
         if instance:
-            kwargs['object_id'] = instance.id
-        scores = cls.objects.filter(**kwargs)
+            kwargs['total_score__object_id'] = instance.id
         stats = []
         for case in cases:
-            case_sum = 0
-            for score in scores:
-                case_sum += score.vote_set.filter(**case).count()
+            case.update(kwargs)
+            case_sum = Vote.objects.filter(**case).count()
             stats.append(case_sum)
         return stats
 
@@ -150,7 +148,7 @@ class Vote(TimeTrackable):
     of Vote instances, don't update it directly."""
     total_score = db.ForeignKey(TotalScore, verbose_name=_("total score"))
     voter = db.ForeignKey(SCORE_VOTER_MODEL, verbose_name=_("voter"))
-    value = db.IntegerField(verbose_name=_("value"), default=1)
+    value = db.IntegerField(verbose_name=_("value"), default=1, db_index=True)
     reason = db.TextField(verbose_name=_("reason"), blank=True, default="")
 
     def __unicode__(self):
