@@ -176,8 +176,9 @@ class TaggableBase(db.Model):
         else:
             # may happen if this object is not available through Model.objects,
             # e.g. the model has a custom manager which filters it out
-            self_stems = set(TagStem.objects.get_queryset_for_model(self.__class__,
-                self))
+            self_stems = set((i[0] for i in
+                TagStem.objects.get_queryset_for_model(self.__class__,
+                self).values_list('name')))
         for obj, s in stems.iteritems():
             if not s & self_stems:
                 # things without a single common tag are not similar at all
@@ -265,7 +266,7 @@ class TagStemManager(db.Manager):
     def get_dictionary(self, model=None, content_type=None, stem=None,
         stems=None, official=False, author=None, language=None):
         """Returns a dictionary of all tagged objects with values being
-        sets of stems for the specified object.
+        sets of raw stems (strings) for the specified object.
 
         This is basically an overly complex implementation that avoids
         duplicating SQL queries. A straight forward version would be::
@@ -277,9 +278,9 @@ class TagStemManager(db.Manager):
         tagged_objects = self.get_content_objects(model=model,
             content_type=content_type, stem=stem, stems=stems,
             official=official, author=author, language=language)
-        return {obj: set(self.get_queryset_for_model(obj.__class__,
-            instance=obj, official=official, author=author, language=language))
-            for obj in tagged_objects}
+        return {obj: set((i[0] for i in self.get_queryset_for_model(
+            obj.__class__, instance=obj, official=official, author=author,
+            language=language).values_list('name'))) for obj in tagged_objects}
 
     def get_content_objects(self, model=None, content_type=None, stem=None,
         stems=None, official=False, author=None, language=None):
@@ -308,14 +309,15 @@ class TagStemManager(db.Manager):
             kwargs["author"] = author
         if language is not None:
             kwargs["language"] = language
-        tagged_cts_oids = set((t.content_type_id, t.object_id)
-            for t in Tag.objects.filter(**kwargs))
+        tags = Tag.objects.filter(**kwargs)
+        tagged_cts_oids = set(tags.values_list('content_type_id',
+            'object_id').distinct())
         if not content_type:
-            tagged_cts = {id for id, _ in tagged_cts_oids}
+            tagged_cts = {t[0] for t in tagged_cts_oids}
             tagged_models = {id: ContentType.objects.get(pk=id).model_class()
                 for id in tagged_cts}
-        return set(self._yield_objects_that_exist(tagged_cts_oids,
-            tagged_models))
+        return self._yield_objects_that_exist(tagged_cts_oids,
+            tagged_models)
 
     def get_queryset_for_model(self, model, instance=None, official=False,
         author=None, language=None):
@@ -347,11 +349,10 @@ class TagStemManager(db.Manager):
         yet) it may be possible that asking for an object that exist
         in the database (and has tags on it) raises DoesNotExist instead.
         Here we silently ignore those."""
-        for ctid, oid in cts_oids:
-            try:
-                yield model_table[ctid].objects.get(pk=oid)
-            except model_table[ctid].DoesNotExist:
-                continue
+        for ctid, table in model_table.iteritems():
+            ids = {o[1] for o in filter(lambda x: x[0] == ctid, cts_oids)}
+            for obj in table.objects.filter(pk__in=ids):
+                yield obj
 
 
 class TagStem(Named.NonUnique, Localized, Taggable.NoDefaultTags):
